﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Reflection;
using System.IO;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.UI;
using System.Windows.Forms;

namespace RCAddInRegisterFiltersCS
{
	[System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
	public partial class AppAddIn
	{
        // コンボボックス固有名
        string comboBoxName = "RcRegisterFilterComboBox";
        // クリアボタン固有名
        string clearButtonName = "RcRegisterFilterClearButton";
        // 削除ボタン固有名
        string deleteButtonName = "RcRegisterFilterDeleteButton";

        // 登録名保存用変数
        string registeringName = "";

        // 条件選択データ
        FilterSettingsData filterSettingsData = null;

        private void AppAddIn_Startup(object sender, EventArgs e)
		{
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

            // XMLファイルから設定を読み込む
            if(!LoadFilterSettingsData(out filterSettingsData))
                filterSettingsData = new FilterSettingsData();// 読み込めなかった場合は初期値

            // ツールバーの初期化
            InitToolBer();
        }

		private void AppAddIn_Shutdown(object sender, EventArgs e)
		{
            // XMLファイルに設定を保存する
            SaveFilterSettingsData(filterSettingsData);
        }

        /// <summary>
        /// ツールバーを取得する
        /// </summary>
        /// <returns></returns>
        public RootPro.RootProCAD.UI.ToolBar GetToolBar()
        {
            // ツールバー名で検索
            RootPro.RootProCAD.UI.ToolBar[] toolbars = this.ToolBars.FindByText(Properties.Resources.ToolBarText);
            if(toolbars != null)
                return toolbars[0];

            return null;
        }

        /// <summary>
        /// コンボボックスを取得する
        /// </summary>
        /// <returns></returns>
        public ToolBarComboBoxItem GetComboBox()
        {
            // ツールバーを取得
            RootPro.RootProCAD.UI.ToolBar toolbar = GetToolBar();
            if(toolbar != null)
            {
                // コンボボックス名で検索
                ToolBarItem[] toolBarButtonItem = toolbar.Items.Find(comboBoxName);
                if(toolBarButtonItem != null && toolBarButtonItem.Length != 0)
                    return (ToolBarComboBoxItem)toolBarButtonItem[0];
            }
            return null;
        }

        /// <summary>
        /// クリアボタンを取得する
        /// </summary>
        /// <returns></returns>
        public ToolBarButtonItem GetClearButton()
        {
            // ツールバーを取得
            RootPro.RootProCAD.UI.ToolBar toolbar = GetToolBar();
            if(toolbar != null)
            {
                // クリアボタン名で検索
                ToolBarItem[] toolBarButtonItem = toolbar.Items.Find(clearButtonName);
                if(toolBarButtonItem != null && toolBarButtonItem.Length != 0)
                    return (ToolBarButtonItem)toolBarButtonItem[0];
            }
            return null;
        }

        /// <summary>
        /// 削除ボタンを取得する
        /// </summary>
        /// <returns></returns>
        public ToolBarButtonItem GetDeleteButton()
        {
            // ツールバーを取得
            RootPro.RootProCAD.UI.ToolBar toolbar = GetToolBar();
            if(toolbar != null)
            {
                // 削除ボタン名で検索
                ToolBarItem[] toolBarButtonItem = toolbar.Items.Find(deleteButtonName);
                if(toolBarButtonItem != null && toolBarButtonItem.Length != 0)
                    return (ToolBarButtonItem)toolBarButtonItem[0];
            }
            return null;
        }

        /// <summary>
        /// ツールバーを初期化
        /// </summary>
        /// <returns></returns>
        public void InitToolBer()
        {
            // ツールバーを取得
            RootPro.RootProCAD.UI.ToolBar toolbar = GetToolBar();

            // ツールバーがなかった場合
            if(toolbar == null)
            {
                // ツールバーの追加
                toolbar = this.ToolBars.Add(Properties.Resources.ToolBarText);

                // コンボボックスを追加
                ToolBarComboBoxItem comboBox = (ToolBarComboBoxItem)toolbar.Items.Add(ToolBarItemType.ComboBox);
                comboBox.DropDownStyle = RootPro.RootProCAD.UI.ComboBoxStyle.DropDownList;

                // 「登録」を追加
                string[] str = { Properties.Resources.ToolBarComboBoxRegisterText };
                comboBox.SetItems(str);
                comboBox.SelectedIndex = -1;
                comboBox.ToolTipText = Properties.Resources.ToolBarComboBoxToolTipText;

                // コンボボックスの固有名セット
                comboBox.Name = comboBoxName;

                // 変更イベント時の関数登録
                comboBox.SelectedIndexChanged += new EventHandler(comboBox_selectedIndexChanged);

                // セパレータを追加
                toolbar.Items.Add(ToolBarItemType.Separator);

                // クリアボタンを追加
                ToolBarButtonItem clearButton = (ToolBarButtonItem)toolbar.Items.Add(ToolBarItemType.Button);

                // クリアボタンの表示をセット
                clearButton.DisplayStyle = ToolBarItemDisplayStyle.Text;
                clearButton.Text = Properties.Resources.ClearButtonText;

                // ツールチップをセット
                clearButton.ToolTipText = Properties.Resources.ClearButtonToolTipText;

                // ボタンの固有名セット
                clearButton.Name = clearButtonName;

                // クリックイベント時の関数登録
                clearButton.Click += new EventHandler(clearButton_Click);

                // 削除ボタンを追加
                ToolBarButtonItem deleteButton = (ToolBarButtonItem)toolbar.Items.Add(ToolBarItemType.Button);

                // クリアボタンの表示をセット
                deleteButton.DisplayStyle = ToolBarItemDisplayStyle.Text;
                deleteButton.Text = Properties.Resources.DeleteButtonText;

                // ツールチップをセット
                deleteButton.ToolTipText = Properties.Resources.DeleteButtonToolTipText;

                // ボタンの固有名セット
                deleteButton.Name = deleteButtonName;

                // クリックイベント時の関数登録
                deleteButton.Click += new EventHandler(deleteButton_Click);
            }
            // ツールバーが登録済み
            else
            {
                // コンボの初期化
                ToolBarComboBoxItem comboBox = GetComboBox();
                if(comboBox != null)
                {
                    // 設定データを取得
                    List<FilterData> filterList = filterSettingsData.GetFilterList();

                    // コンボアイテムリスト
                    List<string> comboboxList = new List<string>();
                    foreach(var filter in filterList)
                        comboboxList.Insert(0, filter.filterName);

                    // 最後に「登録」を追加
                    comboboxList.Add(Properties.Resources.ToolBarComboBoxRegisterText);

                    // コンボを更新する
                    comboBox.SetItems(comboboxList.ToArray());

                    comboBox.SelectedIndex = -1; // 最初の文字を選択

                    // イベントハンドラを登録
                    comboBox.SelectedIndexChanged += new EventHandler(comboBox_selectedIndexChanged);
                }

                // ボタンの初期化
                ToolBarButtonItem clearButton = GetClearButton();
                if(clearButton != null)
                    // クリックイベント時の関数登録
                    clearButton.Click += new EventHandler(clearButton_Click);

                ToolBarButtonItem deleteButton = GetDeleteButton();
                if(deleteButton != null)
                    // クリックイベント時の関数登録
                    deleteButton.Click += new EventHandler(deleteButton_Click);
            }
        }

        /// <summary>
        /// コンボボックスが選択されたときのイベント処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void comboBox_selectedIndexChanged(object sender, EventArgs e)
        {
            // コンボボックスを取得
            ToolBarComboBoxItem combobox = GetComboBox();
            if(combobox != null)
            {
                string[] items = combobox.GetItems();

                // 選択されたインデックス取得
                int selectedIndex = combobox.SelectedIndex;

                // 選択した文字列取得
                string selectedString = "";
                if(selectedIndex != -1)
                    selectedString = items[selectedIndex];

                // 登録が選択された場合
                if(selectedString.Equals(Properties.Resources.ToolBarComboBoxRegisterText))
                {
                    // 名前の入力フォームを表示する
                    RcNameForm nameForm = new RcNameForm();
                    nameForm.Text = Properties.Resources.NameFormText;
                    nameForm.SetInputLabel(Properties.Resources.NameFormInputLabel);
                    nameForm.ShowDialog();

                    if(nameForm.DialogResult == DialogResult.OK)
                    {
                        string nameText = nameForm.GetRegisteredNameText();
                        // 同じ名前のものがある場合はエラー
                        foreach(string item in items)
                        {
                            if(item.Equals(nameText))
                            {
                                combobox.SelectedIndex = -1;
                                MessageBox.Show(Properties.Resources.ErrorSameNameExists);
                                return;
                            }
                        }

                        // 登録名
                        registeringName = nameText;

                        // 条件選択コマンドを取得
                        Command command = CommandManager.Commands[CommandNames.SettingsSelectFilter];
        
                        // コマンド終了時のイベントハンドラを登録
                        command.End += new EventHandler(filterCommand_End);

                        // 条件選択コマンドを実行
                        CommandManager.ExecuteCommand(CommandNames.SettingsSelectFilter);
                    }
                    else
                    {
                        combobox.SelectedIndex = -1;
                    }
                }
                // 登録済みの条件選択名が選択された場合
                else
                {
                    // 選択されたフィルタを取得
                    FilterData selectedFilter = filterSettingsData.GetFilterData(selectedString);

                    if(selectedFilter != null)
                    {
                        // 条件選択オブジェクトを取得
                        var selectionFilter = ActiveDocument.SelectionFilter;

                        // 条件選択を有効にする
                        selectionFilter.Enabled = true;

                        // 図形の条件選択をセット
                        selectionFilter.ShapeTypeFilter = selectedFilter.shapeTypeFilters;

                        // 色の条件選択を取得
                        ColorFilterDictionary colorFilters = selectionFilter.ColorFilters;

                        // 一旦全てOFFにする
                        foreach(var filter in colorFilters)
                            colorFilters[filter.Key] = false;

                        // 選択された設定の色フィルタをONにする
                        foreach(var colorNumber in selectedFilter.colorFilterNumbers)
                        {
                            if(colorNumber >= 1 && colorNumber <= colorFilters.Count)
                                colorFilters[colorFilters[colorNumber - 1].Key] = true;
                        }


                        // 線種の条件選択を取得
                        LinetypeFilterDictionary lineTypeFilters = selectionFilter.LinetypeFilters;

                        // 一旦全てOFFにする
                        foreach(var filter in lineTypeFilters)
                            lineTypeFilters[filter.Key] = false;

                        // 選択された設定の線種フィルタをONにする
                        foreach(var lineTypeNumber in selectedFilter.lineTypeFilterNumbers)
                        {
                            if(lineTypeNumber >= 1 && lineTypeNumber <= lineTypeFilters.Count)
                                lineTypeFilters[lineTypeFilters[lineTypeNumber - 1].Key] = true;
                        }


                        // 線幅の条件選択を取得
                        LinewidthFilterDictionary lineWidthFilters = selectionFilter.LinewidthFilters;

                        // 一旦全てOFFにする
                        foreach(var filter in lineWidthFilters)
                            lineWidthFilters[filter.Key] = false;

                        // 選択された設定の線幅フィルタをONにする
                        foreach(var lineWidthNumber in selectedFilter.lineWidthFilterNumbers)
                        {
                            if(lineWidthNumber >= 1 && lineWidthNumber <= lineWidthFilters.Count)
                                lineWidthFilters[lineWidthFilters[lineWidthNumber - 1].Key] = true;
                        }

                        // 選択したものをコンボボックスの先頭に移動
                        List<string> newItemsList = new List<string>();
                        newItemsList.Add(selectedString);
                        foreach(var item in items)
                        {
                            if(!item.Equals(selectedString))
                                newItemsList.Add(item);
                        }
                        combobox.SetItems(newItemsList.ToArray());
                        combobox.SelectedIndex = 0;

                        // データの順序を変更
                        List<FilterData> newFilterList = new List<FilterData>();
                        List<FilterData> filterList = filterSettingsData.GetFilterList();

                        // 選択したフィルタをリストの最後に移動する
                        foreach(var filterItem in filterList)
                        {
                            if(!filterItem.filterName.Equals(selectedString))
                                newFilterList.Add(filterItem.Clone());
                        }
                        newFilterList.Add(selectedFilter.Clone());

                        // 順番を変更したリストをセット
                        filterSettingsData.SetFilterList(newFilterList);
                    }
                }
            }
        }

        /// <summary>
        /// クリアボタンがクリックされたときのイベント処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void clearButton_Click(object sender, EventArgs e)
        {
            // コンボボックスを取得
            ToolBarComboBoxItem combobox = GetComboBox();
            if(combobox != null)
            {
                // 条件選択オブジェクトを取得
                var selectionFilter = ActiveDocument.SelectionFilter;

                // 条件選択を無効にする
                selectionFilter.Enabled = false;

                // 図形の条件選択をクリア
                selectionFilter.ShapeTypeFilter = 0;

                // 色の条件選択をクリア
                ColorFilterDictionary colorFilter = selectionFilter.ColorFilters;
                foreach(var filter in colorFilter)
                    colorFilter[filter.Key] = false;

                // 線種の条件選択をクリア
                LinetypeFilterDictionary linetypeFilter = selectionFilter.LinetypeFilters;
                foreach(var filter in linetypeFilter)
                    linetypeFilter[filter.Key] = false;

                // 線幅の条件選択をクリア
                LinewidthFilterDictionary linewidthFilter = selectionFilter.LinewidthFilters;
                foreach(var filter in linewidthFilter)
                    linewidthFilter[filter.Key] = false;


                combobox.SelectedIndex = -1;
            }
        }

        /// <summary>
        /// 削除ボタンがクリックされたときのイベント処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void deleteButton_Click(object sender, EventArgs e)
        {
            // コンボボックスを取得
            ToolBarComboBoxItem combobox = GetComboBox();
            if(combobox != null)
            {
                // コンボで選択されているインデックスを取得
                int selectedIndex = combobox.SelectedIndex;
                if(selectedIndex != -1)
                {
                    string[] items = combobox.GetItems();
                    string selectedString = "";
                    if(selectedIndex != -1)
                        selectedString = items[selectedIndex];
     
                    // 新規登録以外の場合
                    if(!selectedString.Equals(Properties.Resources.ToolBarComboBoxRegisterText))
                    {
                        string msg = selectedString + Properties.Resources.DeleteMessage;
                        // 削除メッセージを表示
                        if(MessageBox.Show(msg,
                            Properties.Resources.DeleteCaption,MessageBoxButtons.YesNo) == DialogResult.Yes)
                        {
                            // 設定から削除する
                            filterSettingsData.DeleteFilterData(selectedString);

                            // 新しいアイテムのリスト
                            string[] newComboBoxTexts = new string[items.Length - 1];

                            int index = 0;
                            foreach(string item in items)
                            {
                                // 選択されたもの以外を追加
                                if(!item.Equals(selectedString))
                                    newComboBoxTexts[index++] = item;
                            }

                            // ドロップダウンリスト更新
                            combobox.SetItems(newComboBoxTexts);
                        }
                    }
                    combobox.SelectedIndex = -1;
                }
            }
        }

        /// <summary>
        /// 条件選択コマンドが終了したときの処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void filterCommand_End(object sender, EventArgs e)
        {
            // 現在の条件選択の状態を取得
            var selectionFilter = ActiveDocument.SelectionFilter;

            // 登録するフィルタ
            FilterData filter = new FilterData();
 
            // フィルタ名をセット
            filter.filterName = registeringName;

            // 図形のフィルタをセット
            filter.shapeTypeFilters = selectionFilter.ShapeTypeFilter;

            // 色のフィルタをセット
            ColorFilterDictionary colorFilters = selectionFilter.ColorFilters;
            foreach(var keyValue in colorFilters)
            {
                // 有効な場合は色番号をフィルタに登録
                if(keyValue.Value == true)
                    filter.AddColorFilter(keyValue.Key.Number);
            }

            // 線種のフィルタをセット
            LinetypeFilterDictionary lineTypeFilters = selectionFilter.LinetypeFilters;
            foreach(var keyValue in lineTypeFilters)
            {
                // 有効な場合は線種番号をフィルタに登録
                if(keyValue.Value == true)
                    filter.AddLineTypeFilter(keyValue.Key.Number);
            }

            // 線幅のフィルタをセット
            LinewidthFilterDictionary lineWidthFilters = selectionFilter.LinewidthFilters;
            foreach(var keyValue in lineWidthFilters)
            {
                // 有効な場合は線幅番号をフィルタに登録
                if(keyValue.Value == true)
                    filter.AddLineWidthFilter(keyValue.Key.Number);
            }

            // フィルタを追加する
            filterSettingsData.AddFilterData(filter);

            // コンボボックスを取得
            ToolBarComboBoxItem combobox = GetComboBox();
            if(combobox != null)
            {
                string[] items = combobox.GetItems();

                // コンボボックスにフィルタ名のアイテムを追加する
                string[] newComboBoxTexts = new string[items.Length + 1];

                // リストの先頭に新しい登録名を追加
                newComboBoxTexts[0] = registeringName;

                // 2つ目以降をコピー
                items.CopyTo(newComboBoxTexts, 1);

                // コンボを更新する
                combobox.SetItems(newComboBoxTexts);
                combobox.SelectedIndex = -1;
            }

            // イベントハンドラを解除
            Command command = CommandManager.Commands[CommandNames.SettingsSelectFilter];
            command.End -= new EventHandler(filterCommand_End);
        }

        #region VSTA generated code
        private void InternalStartup()
		{
			this.Startup += new System.EventHandler(AppAddIn_Startup);
			this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
		}
		#endregion
		private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
		{
			AssemblyName assemblyName = new AssemblyName(args.Name);

			foreach(Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
			{
				if(AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
					return assembly;
			}

			if(assemblyName.Name.EndsWith(".resources"))
			{
				if(assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
				{
					string dllPath = Path.GetDirectoryName(this.AddInFullPath);
					dllPath = Path.Combine(dllPath, assemblyName.CultureName);
					dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

					if(File.Exists(dllPath))
					{
						try
						{
							return Assembly.LoadFrom(dllPath);
						}
						catch
						{
						}
					}
				}
			}

			return null;
		}
	}
}
